/**
 * testingey
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

const sys = require('sys');

function _pt(color, message) {
  return (
    '\033[' + color + 'm' +
    message +
    '\033[0m'
  );
}

function Testingey(name) {
  
  this.name = name;
  
  this._tests = {};
  this._total = 0;
  this._setUp = null;
  this._tearDown = null;
}

Testingey.prototype._enforceCB = function (name, cb) {
  if ('function' !== typeof cb) {
    throw new Error('Testingey#' + name + ': expecting a function as an argument');
  }
};

Testingey.prototype.setUp = function (cb) {
  this._enforceCB('setUp', cb);
  this._setUp = cb;
};

Testingey.prototype.tearDown = function (cb) {
  this._enforceCB('tearDown', cb);
  this._tearDown = cb;
};

Testingey.prototype.spec = function (desc, cb) {
  
  this._enforceCB('spec', cb);
  
  if (desc in this._tests) {
    console.warn('Spec [' + desc + '] being overwritten with new test');
  } else {
    ++this._total;
  }

  this._tests[desc] = cb;
};

Testingey.prototype.run = function () {

  var spec = test = null,
      failed = 0;

  console.log('Running test suite [' + this.name + ']');

  for (spec in this._tests) {
    test = this._tests[spec];
    try {
      sys.print(spec, '... ');
      test.call(undefined);
      console.log(_pt(32, 'PASSED'));
    } catch (err) {
      console.log(_pt(31, 'FAILED'));
      ++failed;
    }
  }

  console.log(
    'Finished!\n' +
    this._total + ' tests, ' + failed + ' failed.'
  );
};

/**
 * Factory method
 */
exports.createSuite = function (name) {
  return new Testingey(name);
};
